/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "MouseEvent.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMWindow.h"
#include "gdom/GdomEventTargetNode.h"
#include "gdom/GdomMouseEvent.h"
#include "gdom/GdomUIEvent.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomDOMWindow.h"
#include "gdom/PrivateGdomEventTargetNode.h"
#include "gdom/PrivateGdomMouseEvent.h"
#include "gdom/PrivateGdomUIEvent.h"
#include "gdom/PrivateIntGdomDOMWindow.h"
#include "gdom/PrivateIntGdomMouseEvent.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMMouseEvent::GDOMMouseEvent(PassRefPtr<WebCore::MouseEvent> impl)
    : GDOMUIEvent(impl)
{
}

void
gdom_mouse_event_init_mouse_event (GdomMouseEvent *thiz, gchar * type, gboolean can_bubble, gboolean cancelable, GdomDOMWindow * view, glong detail, glong screen_x, glong screen_y, glong client_x, glong client_y, gboolean ctrl_key, gboolean alt_key, gboolean shift_key, gboolean meta_key, gushort button, GdomEventTargetNode * related_target)
{
    g_return_if_fail (thiz);
    WebCore::MouseEvent * item = (gdom_mouse_event_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (type);
    g_return_if_fail (view);
    g_return_if_fail (related_target);
    WebCore::String _g_type = WebCore::String::fromUTF8(type);
    WebCore::DOMWindow * _g_view = (gdom_dom_window_get_private(view));
    g_return_if_fail (_g_view);
    WebCore::EventTargetNode * _g_related_target = static_cast<WebCore::EventTargetNode*>(gdom_event_target_node_get_private(related_target));
    g_return_if_fail (_g_related_target);
    item->initMouseEvent(_g_type, can_bubble, cancelable, _g_view, detail, screen_x, screen_y, client_x, client_y, ctrl_key, alt_key, shift_key, meta_key, button, _g_related_target);

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "MouseEvent.h"
#include <gdom/GdomMouseEvent.h>
#include <gdom/PrivateGdomMouseEvent.h>
#include <gdom/PrivateIntGdomMouseEvent.h>
#include <gdom/GdomMouseEventprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomMouseEvent, gdom_mouse_event, GDOM_TYPE_UI_EVENT);

/*
struct _GdomMouseEventPrivate {
    class WebCore::MouseEvent *item;
};

#define GDOM_MOUSE_EVENT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_MOUSE_EVENT, GdomMouseEventPrivate))
*/
G_END_DECLS
WebCore::MouseEvent *
gdom_mouse_event_get_private(GdomMouseEvent *request)
{
    GDOMMouseEvent *res;
    res = static_cast<GDOMMouseEvent*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::MouseEvent*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_SCREEN_X,
    PROP_SCREEN_Y,
    PROP_CLIENT_X,
    PROP_CLIENT_Y,
    PROP_CTRL_KEY,
    PROP_SHIFT_KEY,
    PROP_ALT_KEY,
    PROP_META_KEY,
    PROP_BUTTON,
    PROP_RELATED_TARGET,
    PROP_OFFSET_X,
    PROP_OFFSET_Y,
    PROP_X,
    PROP_Y,
    PROP_FROM_ELEMENT,
    PROP_TO_ELEMENT,
};


static void gdom_mouse_event_finalize(GObject* object)
{
    /*GdomMouseEvent* request = GDOM_MOUSE_EVENT(object);*/
    /*GdomMouseEventPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_UI_EVENT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_mouse_event_parent_class)->finalize(object);
}

static void gdom_mouse_event__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomMouseEvent *self = GDOM_MOUSE_EVENT(object);
    WebCore::MouseEvent * gself = (gdom_mouse_event_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_mouse_event__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomMouseEvent *self = GDOM_MOUSE_EVENT(object);
    WebCore::MouseEvent * gself = (gdom_mouse_event_get_private(self));
    switch (prop_id) {
    case PROP_SCREEN_X:
    {
         g_value_set_long (value, gself->screenX());
         break;
    }
    case PROP_SCREEN_Y:
    {
         g_value_set_long (value, gself->screenY());
         break;
    }
    case PROP_CLIENT_X:
    {
         g_value_set_long (value, gself->clientX());
         break;
    }
    case PROP_CLIENT_Y:
    {
         g_value_set_long (value, gself->clientY());
         break;
    }
    case PROP_CTRL_KEY:
    {
         g_value_set_boolean (value, gself->ctrlKey());
         break;
    }
    case PROP_SHIFT_KEY:
    {
         g_value_set_boolean (value, gself->shiftKey());
         break;
    }
    case PROP_ALT_KEY:
    {
         g_value_set_boolean (value, gself->altKey());
         break;
    }
    case PROP_META_KEY:
    {
         g_value_set_boolean (value, gself->metaKey());
         break;
    }
    case PROP_BUTTON:
    {
         g_value_set_uint (value, gself->button());
         break;
    }
    case PROP_RELATED_TARGET:
    {
         PassRefPtr<WebCore::EventTarget> ptr = gself->relatedTarget();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_OFFSET_X:
    {
         g_value_set_long (value, gself->offsetX());
         break;
    }
    case PROP_OFFSET_Y:
    {
         g_value_set_long (value, gself->offsetY());
         break;
    }
    case PROP_X:
    {
         g_value_set_long (value, gself->x());
         break;
    }
    case PROP_Y:
    {
         g_value_set_long (value, gself->y());
         break;
    }
    case PROP_FROM_ELEMENT:
    {
         PassRefPtr<WebCore::Node> ptr = gself->fromElement();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_TO_ELEMENT:
    {
         PassRefPtr<WebCore::Node> ptr = gself->toElement();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_mouse_event_class_init(GdomMouseEventClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_mouse_event_finalize;
    g_klass->set_property = gdom_mouse_event__set_property_fn;
    g_klass->get_property = gdom_mouse_event__get_property_fn;

    g_object_class_install_property (g_klass, PROP_SCREEN_X,
            g_param_spec_long ("screen_x", /* name */
                "mouse_event_screen_x", /* short description */
                "read-only  glong MouseEvent.screen_x", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_SCREEN_Y,
            g_param_spec_long ("screen_y", /* name */
                "mouse_event_screen_y", /* short description */
                "read-only  glong MouseEvent.screen_y", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CLIENT_X,
            g_param_spec_long ("client_x", /* name */
                "mouse_event_client_x", /* short description */
                "read-only  glong MouseEvent.client_x", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CLIENT_Y,
            g_param_spec_long ("client_y", /* name */
                "mouse_event_client_y", /* short description */
                "read-only  glong MouseEvent.client_y", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CTRL_KEY,
            g_param_spec_boolean ("ctrl_key", /* name */
                "mouse_event_ctrl_key", /* short description */
                "read-only  gboolean MouseEvent.ctrl_key", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_SHIFT_KEY,
            g_param_spec_boolean ("shift_key", /* name */
                "mouse_event_shift_key", /* short description */
                "read-only  gboolean MouseEvent.shift_key", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_ALT_KEY,
            g_param_spec_boolean ("alt_key", /* name */
                "mouse_event_alt_key", /* short description */
                "read-only  gboolean MouseEvent.alt_key", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_META_KEY,
            g_param_spec_boolean ("meta_key", /* name */
                "mouse_event_meta_key", /* short description */
                "read-only  gboolean MouseEvent.meta_key", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_BUTTON,
            g_param_spec_uint ("button", /* name */
                "mouse_event_button", /* short description */
                "read-only  gushort MouseEvent.button", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXUINT16, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_RELATED_TARGET,
            g_param_spec_object ("related_target", /* name */
                "mouse_event_related_target", /* short description */
                "read-only  GdomEventTargetNode * MouseEvent.related_target", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_EVENT_TARGET_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_OFFSET_X,
            g_param_spec_long ("offset_x", /* name */
                "mouse_event_offset_x", /* short description */
                "read-only  glong MouseEvent.offset_x", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_OFFSET_Y,
            g_param_spec_long ("offset_y", /* name */
                "mouse_event_offset_y", /* short description */
                "read-only  glong MouseEvent.offset_y", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_X,
            g_param_spec_long ("x", /* name */
                "mouse_event_x", /* short description */
                "read-only  glong MouseEvent.x", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_Y,
            g_param_spec_long ("y", /* name */
                "mouse_event_y", /* short description */
                "read-only  glong MouseEvent.y", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_FROM_ELEMENT,
            g_param_spec_object ("from_element", /* name */
                "mouse_event_from_element", /* short description */
                "read-only  GdomNode * MouseEvent.from_element", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_TO_ELEMENT,
            g_param_spec_object ("to_element", /* name */
                "mouse_event_to_element", /* short description */
                "read-only  GdomNode * MouseEvent.to_element", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomMouseEventPrivate));*/
}

static void gdom_mouse_event_init(GdomMouseEvent* request)
{
    /*GdomMouseEventPrivate* priv = GDOM_MOUSE_EVENT_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

